
package cz.zcu.kiv.db.dao.jdbc;

import java.util.ArrayList;
import java.util.List;

import javax.sql.DataSource;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.namedparam.MapSqlParameterSource;
import org.springframework.jdbc.core.simple.ParameterizedBeanPropertyRowMapper;
import org.springframework.jdbc.core.simple.SimpleJdbcInsert;
import org.springframework.jdbc.core.simple.SimpleJdbcTemplate;
import org.springframework.stereotype.Repository;

import cz.zcu.kiv.db.dao.OsobaDao;
import cz.zcu.kiv.db.dao.StudiumDao;
import cz.zcu.kiv.db.dao.TemaDao;
import cz.zcu.kiv.db.dao.jdbc.model.JdbcTema;
import cz.zcu.kiv.db.dao.jdbc.model.JdbcZaverecnaPrace;
import cz.zcu.kiv.db.model.osoby.Osoba;
import cz.zcu.kiv.db.model.studium.Obor;
import cz.zcu.kiv.db.model.studium.Tema;
import cz.zcu.kiv.db.model.studium.TypPrace;
import cz.zcu.kiv.db.model.studium.ZaverecnaPrace;

/**
 * Implementace rozhrani TemaDao za pouziti Spring Jdbc.
 * Anotace @Repository("temaDao") ulozi instanci teto tridy do IOC kontejneru springu,
 * kde bude k dispozici pod identifikatorem "temaDao".
 * <code>TemaDao tDao = (TemaDao) ctx.getBean("temaDao");</code>
 *  
 * @author Radek Muzika
 */
@Repository("temaDao")
public class TemaDaoImpl implements TemaDao{

    /**
     * SimpleJdbcTemplate umoznuje snadnou manipulaci s databazi. Vice viz.
     * dokumentace springu http://static.springframework.org/spring/docs/2.5.x/reference/jdbc.html#jdbc-SimpleJdbcTemplate
     */
    private SimpleJdbcTemplate simpleJdbcTemplate;
    /**
     * SimpleJdbcInsert umoznuje snadne vkladani dat do databaze a nasledne ziskani
     * automaticky generovaneho primarniho klice (pokud takovy je). Vice viz.
     * dokumentace springu http://static.springframework.org/spring/docs/2.5.x/reference/jdbc.html#jdbc-simple-jdbc-insert-1
     */
    private SimpleJdbcInsert temaInsert;
    private SimpleJdbcInsert typPraceInsert;
    private SimpleJdbcInsert udalostInsert;
    
    /**
     * Anotace @Autowired zajistuje automaticke instanciovani objektu z IOC kontejneru.
     * Coz znamena ze atributu _osobaDao bude v instanci tridy, ulozenem v IOC kontejneru
     * pod identifikatorem "temaDao" automaticky prirazena instance tridy OsobaDao, ktera
     * je jiz ulozena v IOC kontejneru.
     */
    @Autowired
    private OsobaDao _osobaDao;
    @Autowired
    private StudiumDao _studiumDao;
    
    /**
     * Anotace @Autowired pouzita u metody init je dalsim moznym pouzitim IOC. Beana z IOC kontejneru
     * je vlozena do volani metody jako parametr. V tomto pripade se jedna o dataSource,
     * nadefinovany v konfiguracnim souboru.
     * @param dataSource
     */
    @Autowired
	public void init(DataSource dataSource) {
		this.simpleJdbcTemplate = new SimpleJdbcTemplate(dataSource);

		this.temaInsert = new SimpleJdbcInsert(dataSource)
			.withTableName("studijni_prace_temata")
			.usingGeneratedKeyColumns("ID");
		this.typPraceInsert = new SimpleJdbcInsert(dataSource)
			.withTableName("studini_prace_typy")
			.usingGeneratedKeyColumns("id");
		this.udalostInsert = new SimpleJdbcInsert(dataSource)
			.withTableName("studijni_zaverecne_prace")
			.usingGeneratedKeyColumns("id");
	}

    public List<Tema> getTemata() {
        return this.getTemata(false);
    }

    public List<Tema> getTemata(boolean vcetneNeaktivnich) {
        String sql = "select * from studijni_prace_temata "+
                (vcetneNeaktivnich?"":" where aktivni=1")+
                " order by kdy_vypsano desc";
        List<JdbcTema> jdbclst = this.simpleJdbcTemplate.query(
                sql,
                ParameterizedBeanPropertyRowMapper.newInstance(JdbcTema.class));
        List<Tema> lst = new ArrayList<Tema>();
        for(JdbcTema tema: jdbclst) {
            tema.setOsobaDao(_osobaDao);
            tema.setStudiumDao(_studiumDao);
            tema.setTemaDao(this);
            lst.add(tema);
        }
        return lst;
    }

    public List<TypPrace> getTypyPrace() {
        return this.getTypyPrace(false);
    }

    public List<TypPrace> getTypyPrace(boolean vcetneNeaktivnich) {
        String sql = "select * from studijni_prace_typy "+
                (vcetneNeaktivnich?"":" where aktivni=1")+
                " order by zkratka";
        return this.simpleJdbcTemplate.query(
                sql,
                ParameterizedBeanPropertyRowMapper.newInstance(TypPrace.class));
    }

    public List<ZaverecnaPrace> getZaverecnePrace() {
        return this.getZaverecnePrace(false);
    }

    public List<ZaverecnaPrace> getZaverecnePrace(boolean vcetneNeaktivnich) {
        String sql = "select * from studijni_zaverecne_prace "+
                (vcetneNeaktivnich?"":" where aktivni=1")+
                " order by datum_obhajeno desc";
        List<JdbcZaverecnaPrace> jdbclst = this.simpleJdbcTemplate.query(
                sql,
                ParameterizedBeanPropertyRowMapper.newInstance(JdbcZaverecnaPrace.class));
        List<ZaverecnaPrace> lst = new ArrayList<ZaverecnaPrace>();
        for(JdbcZaverecnaPrace prace: jdbclst) {
            prace.setTemaDao(this);
            lst.add(prace);
        }
        return lst;
    }

    public Tema getTema(int id) {
        String sql = "select * from studijni_prace_temata where id = ?";
        try {
            JdbcTema tema = this.simpleJdbcTemplate.queryForObject(
                    sql,
                    ParameterizedBeanPropertyRowMapper.newInstance(JdbcTema.class),
                    id);
            tema.setOsobaDao(_osobaDao);
            tema.setStudiumDao(_studiumDao);
            tema.setTemaDao(this);
            return tema;
        } catch (org.springframework.dao.EmptyResultDataAccessException ex) {
            return null;
        }
    }

    public TypPrace getTypPrace(int id) {
        String sql = "select * from studijni_prace_typy where id = ?";
        try {
            return this.simpleJdbcTemplate.queryForObject(
                    sql,
                    ParameterizedBeanPropertyRowMapper.newInstance(TypPrace.class),
                    id);
        } catch (org.springframework.dao.EmptyResultDataAccessException ex) {
            return null;
        }
    }

    public ZaverecnaPrace getZaverecnaPrace(int id) {
        String sql = "select * from studijni_zaverecne_prace where id = ?";
        try {
            JdbcZaverecnaPrace prace = this.simpleJdbcTemplate.queryForObject(
                    sql,
                    ParameterizedBeanPropertyRowMapper.newInstance(JdbcZaverecnaPrace.class),
                    id);
            prace.setTemaDao(this);
            return prace;
        } catch (org.springframework.dao.EmptyResultDataAccessException ex) {
            return null;
        }
    }

    public void ulozTema(Tema tema) {
        if (tema.isNovyZaznam()) {
            tema.setId(udalostInsert.executeAndReturnKey(
                    this.getTemaParameterSource(tema)).intValue());
        } else {
            String sql = "update studijni_prace_temata set " +
                    "nazev_cz=:nazev_cz, " +
                    "nazev_en=:nazev_en, " +
                    "popis_cz=:popis_cz, " +
                    "popis_en=:popis_en, " +
                    "url=:url, " +
                    "je_externi=:je_externi, " +
                    "ext_vedouci=:ext_vedouci, " +
                    "pro_akad_rok=:pro_akad_rok, " +
                    "kdy_vypsano=:kdy_vypsano, " +
                    "kdy_aktualizovano=:kdy_aktualizovano, " +
                    "kdy_zverejnit=:kdy_zverejnit, " +
                    "kdy_rezervovano=:kdy_rezervovano, " +
                    "je_prideleno=:je_prideleno, " +
                    "datum_dokonceno=:datum_dokonceno, " +
                    "komu_zadano_jmeno=:komu_zadano_jmeno, " +
                    "komu_zadano_oscislo=:komu_zadano_oscislo, " +
                    "komu_zadano_email=:komu_zadano_email, " +
                    "autor_zaznamu=:autor_zaznamu, " +
                    "fk_garant=:fk_garant," +
                    "fk_typ_prace=:fk_typ_prace, " +
                    "fk_obor=:fk_obor " +
                    "where id=:id";
            this.simpleJdbcTemplate.update(
                    sql,
                    this.getTemaParameterSource(tema));
        }
    }
    
    /**
     * MapSqlParameterSource slouzi k namapovani objektu na jmenne parametry sql dotazu
     * (Napriklad: select * from udalosti where nazev_cz=:nazev_cz)
     * @param tema
     * @return
     */
    private MapSqlParameterSource getTemaParameterSource(Tema tema) {
        return new MapSqlParameterSource()
                .addValue("id", tema.getId())
                .addValue("nazev_cz", tema.getNazevCz())
                .addValue("popis_cz", tema.getPopisCz())
                .addValue("nazev_en", tema.getNazevEn())
                .addValue("popis_en", tema.getPopisEn())
                .addValue("url", tema.getUrl())
                .addValue("je_externi", tema.isJeExterni()?1:0)
                .addValue("ext_vedouci", tema.getExtVedouci())
                .addValue("pro_akad_rok", tema.getProAkadRok())
                .addValue("kdy_vypsano", tema.getKdyVypsano())
                .addValue("kdy_aktualizovano", tema.getKdyAktualizovano())
                .addValue("kdy_zverejnit", tema.getKdyZverejit())
                .addValue("kdy_rezervovano", tema.getKdyRezervovano())
                .addValue("je_prideleno", tema.isJePrideleno()?1:0)
                .addValue("datum_dokonceno", tema.getDatumDokonceno())
                .addValue("komu_zadano_jmeno", tema.getKomuZadanoJmeno())
                .addValue("komu_zadano_oscislo", tema.getKomuZadanoOsCislo())
                .addValue("komu_zadano_email", tema.getKomuZadanoEmail())
                .addValue("autor_zaznamu", tema.getZaznamEditor())
                .addValue("fk_garant", tema.getGarant()!=null?
                    tema.getGarant().getId():0)
                .addValue("fk_typ_prace", tema.getTypPrace()!=null?
                    tema.getTypPrace().getId():0)
                .addValue("fk_obor", tema.getObor()!=null?
                    tema.getObor().getId():0);
    }

    public void ulozTypPrace(TypPrace typ) {
        if (typ.isNovyZaznam()) {
            typ.setId(udalostInsert.executeAndReturnKey(
                    this.getTypPraceParameterSource(typ)).intValue());
        } else {
            String sql = "update studijni_prace_typy set " +
                    "nazev_cz=:nazev_cz, " +
                    "nazev_en=:nazev_en, " +
                    "zkratka=:zkratka, " +
                    "je_zaverecna_prace=:je_zaverecna_prace " +
                    "where id=:id";
            this.simpleJdbcTemplate.update(
                    sql,
                    this.getTypPraceParameterSource(typ));
        }
    }

    /**
     * MapSqlParameterSource slouzi k namapovani objektu na jmenne parametry sql dotazu
     * (Napriklad: select * from udalosti where nazev_cz=:nazev_cz)
     * @param typ
     * @return
     */
    private MapSqlParameterSource getTypPraceParameterSource(TypPrace typ) {
        return new MapSqlParameterSource()
                .addValue("id", typ.getId())
                .addValue("nazev_cz", typ.getNazevCz())
                .addValue("nazev_en", typ.getNazevEn())
                .addValue("zkratka", typ.getZkratka())
                .addValue("je_zaverecna_prace", typ.isJeZaverecnaPrace()?1:0);
    }
    
    public void ulozZaverecnouPraci(ZaverecnaPrace prace) {
        if (prace.isNovyZaznam()) {
            prace.setId(udalostInsert.executeAndReturnKey(
                    this.getZaverecnaPraceParameterSource(prace)).intValue());
        } else {
            String sql = "update studijni_zaverecne_prace set " +
                    "fk_tema=:fk_tema, " +
                    "datum_obhajeno=:datum_obhajeno, " +
                    "autor_zaznamu=:autor_zaznamu " +
                    "where id=:id";
            this.simpleJdbcTemplate.update(
                    sql,
                    this.getZaverecnaPraceParameterSource(prace));
        }
    }
    
    /**
     * MapSqlParameterSource slouzi k namapovani objektu na jmenne parametry sql dotazu
     * (Napriklad: select * from udalosti where nazev_cz=:nazev_cz)
     * @param prace
     * @return
     */
    private MapSqlParameterSource getZaverecnaPraceParameterSource(ZaverecnaPrace prace) {
        return new MapSqlParameterSource()
                .addValue("id", prace.getId())
                .addValue("fk_tema", prace.getTema()!=null?
                    prace.getTema().getId():0)
                .addValue("datum_obhajeno", prace.getDatumObhajeno())
                .addValue("autor_zaznamu", prace.getZaznamEditor());
    }

    /**
     * Smaze zaznam z databaze.
     * @param tema
     */
    public void smazTema(Tema tema) {
        String sql = "delete from studijni_prace_temata where id=?";
        this.simpleJdbcTemplate.update(sql, tema.getId());
    }

    /**
     * Smaze zaznam z databaze.
     * @param typ
     */
    public void smazTypPrace(TypPrace typ) {
        String sql = "delete from studijni_prace_typy where id=?";
        this.simpleJdbcTemplate.update(sql, typ.getId());
    }

    /**
     * Smaze zaznam z databaze.
     * @param prace
     */
    public void smazZaverecnouPrace(ZaverecnaPrace prace) {
        String sql = "delete from studijni_zaverecne_prace where id=?";
        this.simpleJdbcTemplate.update(sql, prace.getId());
    }

    public List<Tema> getTemataDleOsoby(Osoba osoba) {
        return this.getTemataDleOsoby(osoba, false);
    }

    public List<Tema> getTemataDleOsoby(Osoba osoba, boolean vcetneNeaktivnich) {
        String sql = "select * from studijni_prace_temata " +
                "where fk_garant=?"+
                (vcetneNeaktivnich?"":" and aktivni=1")+
                " order by kdy_vypsano desc";
        List<JdbcTema> jdbclst = this.simpleJdbcTemplate.query(
                sql,
                ParameterizedBeanPropertyRowMapper.newInstance(JdbcTema.class),
                osoba.getId());
        List<Tema> lst = new ArrayList<Tema>();
        for(JdbcTema tema: jdbclst) {
            tema.setOsobaDao(_osobaDao);
            tema.setStudiumDao(_studiumDao);
            tema.setTemaDao(this);
            lst.add(tema);
        }
        return lst;
    }

    public List<Tema> getTemataDleOboru(Obor obor) {
        return this.getTemataDleOboru(obor, false);
    }

    public List<Tema> getTemataDleOboru(Obor obor, boolean vcetneNeaktivnich) {
        String sql = "select * from studijni_prace_temata " +
                "where fk_garant=?"+
                (vcetneNeaktivnich?"":" and aktivni=1")+
                " order by kdy_vypsano desc";
        List<JdbcTema> jdbclst = this.simpleJdbcTemplate.query(
                sql,
                ParameterizedBeanPropertyRowMapper.newInstance(JdbcTema.class),
                obor.getId());
        List<Tema> lst = new ArrayList<Tema>();
        for(JdbcTema tema: jdbclst) {
            tema.setOsobaDao(_osobaDao);
            tema.setStudiumDao(_studiumDao);
            tema.setTemaDao(this);
            lst.add(tema);
        }
        return lst;
    }

    public List<Tema> getTemataDleTypu(TypPrace typ) {
        return this.getTemataDleTypu(typ, false);
    }

    public List<Tema> getTemataDleTypu(TypPrace typ, boolean vcetneNeaktivnich) {
        String sql = "select * from studijni_prace_temata " +
                "where fk_garant=?"+
                (vcetneNeaktivnich?"":" and aktivni=1")+
                " order by kdy_vypsano desc";
        List<JdbcTema> jdbclst = this.simpleJdbcTemplate.query(
                sql,
                ParameterizedBeanPropertyRowMapper.newInstance(JdbcTema.class),
                typ.getId());
        List<Tema> lst = new ArrayList<Tema>();
        for(JdbcTema tema: jdbclst) {
            tema.setOsobaDao(_osobaDao);
            tema.setStudiumDao(_studiumDao);
            tema.setTemaDao(this);
            lst.add(tema);
        }
        return lst;
    }

    /**
     * Nastavi sloupec "aktivni" u daneho zaznamu na hodnotu 0
     * @param tema
     */
    public void deaktivujTema(Tema tema) {
        String sql = "update studijni_prace_temata set aktivni=0 where id=:id";
        this.simpleJdbcTemplate.update(sql, tema.getId());
    }

    /**
     * Nastavi sloupec "aktivni" u daneho zaznamu na hodnotu 0
     * @param typ
     */
    public void deaktivujTypPrace(TypPrace typ) {
        String sql = "update studijni_prace_typy set aktivni=0 where id=:id";
        this.simpleJdbcTemplate.update(sql, typ.getId());
    }

    /**
     * Nastavi sloupec "aktivni" u daneho zaznamu na hodnotu 0
     * @param prace
     */
    public void deaktivujZaverecnouPraci(ZaverecnaPrace prace) {
        String sql = "update studijni_zaverecne_prace set aktivni=0 where id=:id";
        this.simpleJdbcTemplate.update(sql, prace.getId());
    }
    

}
